// Shell Stub
// shellstubmain.cpp
// By Bryce Bigwood 1998
// public domain
// see pp. 441-443 of Advanced Programming in the Unix Environment, W. Richard Stevens

#include <unistd.be.h>
#include <signal.h>

static	void	err_sys(char *inChar)
{
	printf("%s\n", inChar);
}

static	void	err_msg(char *inChar)
{
	err_sys(inChar);
}

static void sig_pipe(int signo)
{
	printf("SIGPIPE caught\n");
	exit(1);
}

int main(int argc, char *argv[])
{
	// argv[0]: path of this binary
	// argv[1]: path of binary to "mutate" into (can be sh or any other shell tool, e.g. ftp, or any binary that launches with no 
					// arguments)
	// argv[2]: file descriptor of read end of pipe 1
	// argv[3]: file descriptor of write end of pipe 1
	// argv[4]: file descriptor of read end of pipe 2
	// argv[5]: file descriptor of write end of pipe 2
	
	int		n, fd1[2], fd2[2];
	pid_t	pid;
	
	//printf("shell stub: %d\n", argc);
	//printf("%s\n", argv[0]);
	//printf("%s\n", argv[1]);
	//printf("%s\n", argv[2]);
	//printf("%s\n", argv[3]);
	//printf("%s\n", argv[4]);
	
	// get the passed-in file descriptors
	fd1[0] = atoi(argv[2]);
	fd1[1] = atoi(argv[3]);
	fd2[0] = atoi(argv[4]);
	fd2[1] = atoi(argv[5]);
	//printf("shell stub: fd1[0] = %d, fd1[1] = %d, fd2[0] = %d, fd2[1] = %d\n", fd1[0], fd1[1], fd2[0], fd2[1]);
	
	// close write end of pipe 1
	close(fd1[1]);
	// close read end of pipe 2
	close(fd2[0]);
	// change read end of pipe 1 to Standard In
	if (fd1[0] != STDIN_FILENO)
	{
		if (dup2(fd1[0], STDIN_FILENO) != STDIN_FILENO)
			err_sys("dup2 error to stdin");
		close(fd1[0]);
	}
	// change write end of pipe 2 to Standard Out
	if (fd2[1] != STDOUT_FILENO)
	{
		if (dup2(fd2[1], STDOUT_FILENO) != STDOUT_FILENO)
			err_sys("dup2 error to stdout");
		close(fd2[1]);
	}

 	char *args[2]; 
    args[0] = strdup(argv[1]); // binary to "mutate" into e.g. "/bin/sh"; strdup to be safe when deallocating char pointers
    args[1] = NULL;


    // "mutate" this process into sh
    execve(args[0], args, environ);

	// if no error occured, we will never get this far
    err_sys("execve error");
 		
}
	
